from rest_framework_simplejwt.tokens import AccessToken
from django.contrib.auth import get_user_model
from ninja.security import HttpBearer
from ninja.errors import HttpError
from .models import UserTokenBlock

User = get_user_model()


class TokenAuth(HttpBearer):
    def authenticate(self, request, token):
        # Validate the access token
        try:
            access_token = AccessToken(token)
        except Exception:
            raise HttpError(401, "Invalid or expired token")

        # Reject tokens issued before password change
        user_id = access_token["user_id"]
        try:
            block = UserTokenBlock.objects.get(user_id=user_id)
        except UserTokenBlock.DoesNotExist:
            pass
        else:
            if access_token["iat"] < int(block.blocked_at.timestamp()):
                raise HttpError(401, "Token has been revoked")

        # Retrieve and return the user
        try:
            user = User.objects.get(id=user_id)
        except User.DoesNotExist:
            raise HttpError(401, "Invalid or expired token")
        return user
