from typing import Optional, List, Literal
from ninja import Schema
from pydantic import Field


class SetSerial(Schema):
    serial_key: str


class ServerSchema(Schema):
    server_id: str
    name: str
    ping: int
    selected: bool
    ip_address: Optional[str] = None


class ConfigSchema(Schema):
    ip: str
    location: str
    dns: str
    gateway: str
    remaining_days: Optional[int]
    selected_server: Optional[ServerSchema]
    is_activated: Optional[bool]
    is_connected: Optional[bool]


class UpdateConfigSchema(Schema):
    ip: Optional[str]
    location: Optional[str]
    dns: Optional[str]
    gateway: Optional[str]
    selected_server_id: Optional[str]
    is_activated: Optional[bool]
    is_connected: Optional[bool]


class SelectServerSchema(Schema):
    server_id: str


class StartServiceResponse(Schema):
    message: str
    output: Optional[str]
    error: Optional[str]


class DisconnectServiceResponse(Schema):
    message: str
    output: Optional[str] = None
    error: Optional[str] = None


class RestartServiceResponse(Schema):
    message: str
    output: Optional[str] = None
    error: Optional[str] = None


class AuthSchema(Schema):
    username: str
    password: str


class AuthResponse(Schema):
    access: str
    refresh: str


class UpdateUserSchema(Schema):
    current_password: str
    username: Optional[str] = None
    password: Optional[str] = None


class HostapdConfig(Schema):
    interface: Optional[str]
    driver: Optional[str]
    ssid: Optional[str]
    hw_mode: Optional[str]  # a, b, g (для диапазона сети)
    channel: Optional[int]
    wmm_enabled: Optional[bool]
    macaddr_acl: Optional[int]
    auth_algs: Optional[int]
    ignore_broadcast_ssid: Optional[bool]
    wpa: Optional[int]
    wpa_passphrase: Optional[str]
    wpa_key_mgmt: Optional[str]
    rsn_pairwise: Optional[str]
    network_mode: Optional[str]  # Сетевой режим (например, b/g/n)
    transmit_power: Optional[int]  # Мощность передатчика
    encryption: Optional[str]  # Тип шифрования (WPA2, WPA3 и т.д.)
    frequency: Optional[str]
    enabled: Optional[bool]


class NetworkConfigSchema(Schema):
    interface: Optional[str]
    hw_mode: Optional[str]
    channel: Optional[int]
    wpa: Optional[int]
    wpa_passphrase: Optional[str]


class WiFiStatusSchema(Schema):
    id: int
    ssid: str
    band: str
    frequency: str
    transmit_power: float
    hw_mode: str
    enabled: bool
    channel: str
    wpa_passphrase: str
    network_config: Optional[NetworkConfigSchema] = None


class UpdateNetworkSchema(Schema):
    ssid: str | None = None
    hw_mode: str | None = None
    channel: int | None = None
    wpa_passphrase: str | None = None
    enabled: Optional[bool]


class UpdateWiFiStatusSchema(Schema):
    enabled: bool


class WorkProfileSchema(Schema):
    id: int
    name: str
    profile_type: str
    is_active: bool
    selected_urls: List[str]
    selected_apps: List[str]


class CreateProfileSchema(Schema):
    name: str
    profile_type: Literal["no_restriction", "selection_mode", "exclusion_mode"]
    is_active: bool
    selected_urls: List[str]
    selected_apps: List[str]


class ProfileURLSchema(Schema):
    id: int
    url: str


class ApplicationSchema(Schema):
    id: int
    name: str


class PackageUpdateSchema(Schema):
    package: str
    current_version: str
    new_version: str
    category: str


class UpdateStatusResponse(Schema):
    updates_available: bool
    updates_list: List[PackageUpdateSchema]
    kelvpn_version: str
    backend_version: str


class UpdateStartRequest(Schema):
    update_type: Literal["all", "kelvpn", "system", "custom"] = "all"
    package_list: Optional[List[str]] = None


class UpdateStartResponse(Schema):
    success: bool
    message: str


class UpdateStatusInfoResponse(Schema):
    status: str
    log_file: str
    started_at: Optional[str]
    completed_at: Optional[str]


class UpdateLogResponse(Schema):
    log_content: str


class UpdateResponse(Schema):
    success: bool
    output: str


class UpdateItem(Schema):
    package: str
    current: str
    candidate: str


class UpdateCheckResponse(Schema):
    updates_available: bool
    kelvpn_version: str
    backend_version: str
    kelvpn_ap_version: str
    updates: List[UpdateItem]
    system_updates: List[UpdateItem] = Field(default_factory=list)


class UpdateCustomRequest(Schema):
    packages: List[str]
