import inspect
import logging

logger = logging.getLogger(__name__)


# --- Decorators for SocketMonitorService Handlers ---


def register_socket_handlers(cls):
    """Class decorator to find and register command handlers."""
    cls._handlers = {}
    for name, member in inspect.getmembers(cls):
        if callable(member) and hasattr(member, "_command_to_handle"):
            command = member._command_to_handle
            cls._handlers[command] = member
            logger.debug(f"Registered handler for '{command}': {name}")
    return cls


def command_handler(command_name: str):
    """Method decorator to mark a function as a handler for a command."""

    def decorator(fn):
        fn._command_to_handle = command_name
        return fn

    return decorator
